import SwiftUI
import HealthKit

struct ContentView: View {
    
    @StateObject var database: HealthDatabase
    
    @State var navigationPath: NavigationPath = .init()
    
    var body: some View {
        NavigationStack(path: $navigationPath) {
            VStack {
                List {
                    ForEach(database.workouts) { workout in
                        NavigationLink(value: workout) {
                            VStack(alignment: .leading) {
                                Text(workout.typeString)
                                Text(workout.dateString)
                                    .font(.caption)
                                    .foregroundStyle(.secondary)
                            }
                        }
                    }
                }
            }
            .navigationTitle("Workouts")
            .navigationDestination(for: Workout.self) {
                WorkoutDetailView(workout: $0)
                    .environmentObject(database)
            }
        }
    }
}

#Preview {
    ContentView(database: .mock())
}
