import Foundation
import SQLite
import CoreLocation

typealias Database = Connection

final class HealthDatabase: ObservableObject {
    
    private let fileUrl: URL

    private let database: Connection

    @Published
    var workouts: [Workout] = []
    
    convenience init(fileUrl: URL) throws {
        let database = try Connection(fileUrl.path)
        self.init(fileUrl: fileUrl, database: database)
    }

    init(fileUrl: URL, database: Connection) {
        self.fileUrl = fileUrl
        self.database = database
        DispatchQueue.global().async {
            self.readAllWorkouts()
        }
    }

    func readAllWorkouts() {
        let workouts: [Workout]
        do {
            workouts = try Workout.readAll(in: database)
        } catch {
            print("Failed to read workouts: \(error)")
            return
        }
        DispatchQueue.main.async {
            self.workouts = workouts
        }
    }

    func locationSamples(for activity: WorkoutActivity) throws -> [LocationSample] {
        try activity.locationSamples(in: database)
    }

    var activities: [WorkoutActivity] {
        workouts.map { $0.activities }.joined().sorted()
    }

    private func testActivityOverlap() {
        let activities = self.activities
        var current = activities.first!
        for next in activities.dropFirst() {
            let overlap = next.startDate.timeIntervalSince(current.endDate)
            if overlap < 0 {
                print("Overlap \(-overlap.roundedInt) s:")
                print("    Activity \(current.activityType.description): \(current.startDate.timeAndDateText) -> \(current.endDate.timeAndDateText)")
                print("    Activity \(next.activityType.description): \(next.startDate.timeAndDateText) -> \(next.endDate.timeAndDateText)")
            }
            current = next
        }
    }

    convenience init(database: Database) {
        self.init(fileUrl: .init(filePath: "/"), database: database)
    }
}
