import Foundation
import SQLite
import CoreLocation

typealias LocationSample = CLLocation

extension LocationSample {

    private static let table = Table("location_series_data")

    /// `location_series_data[series_identifier]` <-> `workout_activities[ROW_ID]`
    private static let rowSeriesIdentifier = Expression<Int>("series_identifier")

    private static let rowTimestamp = Expression<Double>("timestamp")

    private static let rowLongitude = Expression<Double>("longitude")

    private static let rowLatitude = Expression<Double>("latitude")

    private static let rowAltitude = Expression<Double>("altitude")

    private static let rowSpeed = Expression<Double>("speed")

    private static let rowCourse = Expression<Double>("course")

    private static let rowHorizontalAccuracy = Expression<Double>("horizontal_accuracy")

    private static let rowVerticalAccuracy = Expression<Double>("vertical_accuracy")

    private static let rowSpeedAccuracy = Expression<Double>("speed_accuracy")

    private static let rowCourseAccuracy = Expression<Double>("course_accuracy")

    private static let rowSignalEnvironment = Expression<Double>("signal_environment")

    static func locationSamples(for seriesId: Int, in database: Database) throws -> [LocationSample] {
        try database.prepare(table.filter(rowSeriesIdentifier == seriesId)).map(location)
    }

    static func locationSampleCount(for seriesId: Int, in database: Database) throws -> Int {
        try database.scalar(table.filter(rowSeriesIdentifier == seriesId).count)
    }

    static func locationSamples(from start: Date, to end: Date, in database: Database) throws -> [LocationSample] {
        let startTime = start.timeIntervalSinceReferenceDate
        let endTime = end.timeIntervalSinceReferenceDate
        return try database.prepare(table.filter(rowTimestamp >= startTime && rowTimestamp <= endTime)).map(location)
    }

    static func locationSampleCount(from start: Date, to end: Date, in database: Database) throws -> Int {
        let startTime = start.timeIntervalSinceReferenceDate
        let endTime = end.timeIntervalSinceReferenceDate
        return try database.scalar(table.filter(rowTimestamp >= startTime && rowTimestamp <= endTime).count)
    }

    private static func location(row: Row) -> LocationSample {
        .init(
            coordinate: .init(
                latitude: row[rowLatitude],
                longitude: row[rowLongitude]),
            altitude: row[rowAltitude],
            horizontalAccuracy: row[rowHorizontalAccuracy],
            verticalAccuracy: row[rowHorizontalAccuracy],
            course: row[rowCourse],
            courseAccuracy: row[rowCourseAccuracy],
            speed: row[rowSpeed],
            speedAccuracy: row[rowSpeedAccuracy],
            timestamp: .init(timeIntervalSinceReferenceDate: row[rowTimestamp]),
            sourceInfo: .init())
    }
}
