import Foundation
import SQLite

extension Metadata.Key {

    private static let table = Table("metadata_keys")

    private static let columnId = Expression<Int>("ROWID")

    private static let columnKey = Expression<String>("key")

    static func key(for keyId: Int, in database: Connection) throws -> String {
        try database.prepare(table.filter(columnId == keyId).limit(1)).map { $0[columnKey] }.first!
    }

    static func readAll(in database: Connection) throws -> [Int : Self] {
        try database.prepare(table).reduce(into: [:]) {  dict, row in
            dict[row[columnId]] = .init(rawValue: row[columnKey])
        }
    }

    static func createTable(in database: Connection) throws {
        //try database.execute("CREATE TABLE metadata_keys (ROWID INTEGER PRIMARY KEY AUTOINCREMENT, key TEXT UNIQUE)")
        try database.run(table.create { table in
            table.column(columnId, primaryKey: .autoincrement)
            table.column(columnKey, unique: true)
        })
    }

    static func hasKey(_ key: Self, in database: Connection) throws -> Int? {
        try database.prepare(table.filter(columnKey == key.rawValue).limit(1)).map { $0[columnId] }.first
    }

    static func insert(key: Self, in database: Connection) throws -> Int {
        Int(try database.run(table.insert(columnKey <- key.rawValue)))
    }

}
