import Foundation
import Collections

private let df: DateFormatter = {
    let df = DateFormatter()
    df.timeZone = .current
    df.dateStyle = .short
    df.timeStyle = .short
    return df
}()

struct Workout {
    
    let id: Int
    
    /// The distance in km (?)
    let totalDistance: Double?
    
    let goalType: Int?
    
    let goal: Double?
    
    let condenserVersion: Int?
    
    let condenserDate: Date?
    
    let events: [WorkoutEvent]
    
    let activities: [WorkoutActivity]
    
    let metadata: OrderedDictionary<Metadata.Key, Metadata.Value>

    var firstActivityDate: Date? {
        activities.map { $0.startDate }.min()
    }
    
    var firstEventDate: Date? {
        events.map { $0.date }.min()
    }
    
    var firstAvailableDate: Date? {
        [condenserDate, firstEventDate, firstActivityDate].compactMap { $0 }.min()
    }
    
    var dateString: String {
        guard let firstAvailableDate else {
            return "No date"
        }
        return df.string(from: firstAvailableDate)
    }
    
    var typeString: String {
        activities.first?.activityType.description ?? "Unknown activity"
    }
    
    init(id: Int, totalDistance: Double? = nil, goalType: Int? = nil, goal: Double? = nil, condenserVersion: Int? = nil, condenserDate: Date? = nil, events: [WorkoutEvent] = [], activities: [WorkoutActivity] = [], metadata: [Metadata.Key : Metadata.Value] = [:]) {
        self.id = id
        self.totalDistance = totalDistance
        self.goalType = goalType
        self.goal = goal
        self.condenserVersion = condenserVersion
        self.condenserDate = condenserDate
        self.events = events
        self.activities = activities
        self.metadata = .init(uniqueKeys: metadata.keys, values: metadata.values)
    }
}

extension Workout: Identifiable { }

extension Workout: Equatable {
    static func == (lhs: Workout, rhs: Workout) -> Bool {
        lhs.id == rhs.id
    }
}

extension Workout: Hashable {
    
    func hash(into hasher: inout Hasher) {
        hasher.combine(id)
    }
}
