import Foundation
import SQLite

extension WorkoutActivity {

    private static let table = Table("workout_activities")

    private static let columnId = Expression<Int>("ROWID")

    private static let columnUUID = Expression<Data>("uuid")

    private static let columnOwnerId = Expression<Int>("owner_id")

    private static let columnIsPrimaryActivity = Expression<Bool>("is_primary_activity")

    private static let columnActivityType = Expression<Int>("activity_type")

    private static let columnLocationType = Expression<Int>("location_type")

    private static let columnSwimmingLocationType = Expression<Int>("swimming_location_type")

    private static let columnLapLength = Expression<Data?>("lap_length")

    private static let columnStartDate = Expression<Double>("start_date")

    private static let columnEndDate = Expression<Double>("end_date")

    private static let columnDuration = Expression<Double>("duration")

    private static let columnMetadata = Expression<Data?>("metadata")

    private static func readAll(in database: Connection) throws -> [Self] {
        try database.prepare(table).map(from)
    }

    private static func from(row: Row) -> WorkoutActivity {
        .init(
            id: row[columnId],
            uuid: row[columnUUID],
            isPrimaryActivity: row[columnIsPrimaryActivity],
            activityType: .init(rawValue: UInt(row[columnActivityType]))!,
            locationType: .init(rawValue: row[columnLocationType])!,
            swimmingLocationType: .init(rawValue: row[columnSwimmingLocationType])!,
            lapLength: row[columnLapLength],
            startDate: Date(timeIntervalSinceReferenceDate: row[columnStartDate]),
            endDate: Date(timeIntervalSinceReferenceDate: row[columnEndDate]),
            duration: row[columnDuration],
            metadata: row[columnMetadata])
    }

    static func activities(for workoutId: Int, in database: Connection) throws -> [Self] {
        try database.prepare(table.filter(columnOwnerId == workoutId)).map(from)
    }

    static func createTable(in database: Connection) throws {
        //try database.execute("CREATE TABLE workout_activities (ROWID INTEGER PRIMARY KEY AUTOINCREMENT, uuid BLOB UNIQUE NOT NULL, owner_id INTEGER NOT NULL REFERENCES workouts(data_id) ON DELETE CASCADE, is_primary_activity INTEGER NOT NULL, activity_type INTEGER NOT NULL, location_type INTEGER NOT NULL, swimming_location_type INTEGER NOT NULL, lap_length BLOB, start_date REAL NOT NULL, end_date REAL NOT NULL, duration REAL NOT NULL, metadata BLOB)")
        try database.run(table.create { t in
            t.column(columnId, primaryKey: .autoincrement)
            t.column(columnUUID)
            t.column(columnOwnerId, references: Table("workouts"), Expression<Int>("data_id"))
            t.column(columnIsPrimaryActivity)
            t.column(columnActivityType)
            t.column(columnLocationType)
            t.column(columnSwimmingLocationType)
            t.column(columnLapLength)
            t.column(columnStartDate)
            t.column(columnEndDate)
            t.column(columnDuration)
            t.column(columnMetadata)
        })
    }

    func insert(in database: Connection, dataId: Int) throws {
        try WorkoutActivity.insert(self, dataId: dataId, in: database)
    }

    private static func insert(_ element: WorkoutActivity, dataId: Int, in database: Connection) throws {
        try database.run(table.insert(
            columnUUID <- element.uuid,
            columnOwnerId <- dataId,
            columnIsPrimaryActivity <- element.isPrimaryActivity,
            columnActivityType <- Int(element.activityType.rawValue),
            columnLocationType <- element.locationType.rawValue,
            columnSwimmingLocationType <- element.swimmingLocationType.rawValue,
            columnLapLength <- element.lapLength,
            columnStartDate <- element.startDate.timeIntervalSinceReferenceDate,
            columnEndDate <- element.endDate.timeIntervalSinceReferenceDate,
            columnDuration <- element.duration,
            columnMetadata <- element.metadata)
        )
    }

}
