import Foundation
import HealthKit

struct WorkoutActivity {

    private let id: Int

    let uuid: Data
    
    let isPrimaryActivity: Bool
    
    let activityType: HKWorkoutActivityType
    
    let locationType: HKWorkoutSessionLocationType
    
    let swimmingLocationType: HKWorkoutSwimmingLocationType
    
    let lapLength: Data?
    
    #warning("Fix timezone for dates")
    let startDate: Date
    
    let endDate: Date
    
    let duration: TimeInterval
    
    let metadata: Data?

    init(id: Int, uuid: Data, isPrimaryActivity: Bool, activityType: HKWorkoutActivityType, locationType: HKWorkoutSessionLocationType, swimmingLocationType: HKWorkoutSwimmingLocationType, lapLength: Data?, startDate: Date, endDate: Date, duration: TimeInterval, metadata: Data?) {
        self.id = id
        self.uuid = uuid
        self.isPrimaryActivity = isPrimaryActivity
        self.activityType = activityType
        self.locationType = locationType
        self.swimmingLocationType = swimmingLocationType
        self.lapLength = lapLength
        self.startDate = startDate
        self.endDate = endDate
        self.duration = duration
        self.metadata = metadata
    }

    func locationSamples(in database: Database) throws -> [LocationSample] {
        try LocationSample.locationSamples(from: startDate, to: endDate, in: database)
    }

    func locationSampleCount(in database: Database) throws -> Int {
        try LocationSample.locationSampleCount(from: startDate, to: endDate, in: database)
    }

    func sampleCount(in database: Database) throws -> Int {
        try Sample.sampleCount(from: startDate, to: endDate, in: database)
    }

    func samples(in database: Database) throws -> [SampleDataType : Sample] {
        try Sample.samples(from: startDate, to: endDate, in: database).reduce(into: [:]) {
            $0[$1.dataType] = $1
        }
    }
}

extension WorkoutActivity: Equatable {
    
    static func == (lhs: WorkoutActivity, rhs: WorkoutActivity) -> Bool {
        lhs.id == rhs.id
    }
}

extension WorkoutActivity: Comparable {
    
    static func < (lhs: WorkoutActivity, rhs: WorkoutActivity) -> Bool {
        lhs.startDate < rhs.startDate
    }
}

extension WorkoutActivity: Hashable {
    
    func hash(into hasher: inout Hasher) {
        hasher.combine(id)
    }
}
